Re: [PATCH] c++: don't ICE on NAMESPACE_DECL inside FUNCTION_DECL

2021-12-17 Thread Jason Merrill via Gcc-patches

On 12/15/21 03:53, Matthias Kretz wrote:

OK for trunk? This fixes several modules.exp failures for me.


OK.


── ✂ ──

Code like
   void swap() {
 namespace __variant = __detail::__variant;
 ...
   }
create a NAMESPACE_DECL where the CP_DECL_CONTEXT is a FUNCTION_DECL.
DECL_TEMPLATE_INFO fails on NAMESPACE_DECL and therefore must be handled
first in the assertion.

Signed-off-by: Matthias Kretz 

gcc/cp/ChangeLog:

* module.cc (trees_out::get_merge_kind): NAMESPACE_DECLs also
cannot have a DECL_TEMPLATE_INFO.
---
  gcc/cp/module.cc | 5 +++--
  1 file changed, 3 insertions(+), 2 deletions(-)


--
──
  Dr. Matthias Kretz   https://mattkretz.github.io
  GSI Helmholtz Centre for Heavy Ion Research   https://gsi.de
  stdₓ::simd
──




[PATCH] c++: don't ICE on NAMESPACE_DECL inside FUNCTION_DECL

2021-12-15 Thread Matthias Kretz
OK for trunk? This fixes several modules.exp failures for me.

── ✂ ──

Code like
  void swap() {
namespace __variant = __detail::__variant;
...
  }
create a NAMESPACE_DECL where the CP_DECL_CONTEXT is a FUNCTION_DECL.
DECL_TEMPLATE_INFO fails on NAMESPACE_DECL and therefore must be handled
first in the assertion.

Signed-off-by: Matthias Kretz 

gcc/cp/ChangeLog:

* module.cc (trees_out::get_merge_kind): NAMESPACE_DECLs also
cannot have a DECL_TEMPLATE_INFO.
---
 gcc/cp/module.cc | 5 +++--
 1 file changed, 3 insertions(+), 2 deletions(-)


--
──
 Dr. Matthias Kretz   https://mattkretz.github.io
 GSI Helmholtz Centre for Heavy Ion Research   https://gsi.de
 stdₓ::simd
──diff --git a/gcc/cp/module.cc b/gcc/cp/module.cc
index 3b1b5ca0ac0..2b5a32695d2 100644
--- a/gcc/cp/module.cc
+++ b/gcc/cp/module.cc
@@ -10067,9 +10067,10 @@ trees_out::get_merge_kind (tree decl, depset *dep)
   tree ctx = CP_DECL_CONTEXT (decl);
   if (TREE_CODE (ctx) == FUNCTION_DECL)
 	{
-	  /* USING_DECLs cannot have DECL_TEMPLATE_INFO -- this isn't
-	 permitting them to have one.   */
+	  /* USING_DECLs and NAMESPACE_DECLs cannot have DECL_TEMPLATE_INFO --
+	 this isn't permitting them to have one.   */
 	  gcc_checking_assert (TREE_CODE (decl) == USING_DECL
+			   || TREE_CODE (decl) == NAMESPACE_DECL
 			   || !DECL_LANG_SPECIFIC (decl)
 			   || !DECL_TEMPLATE_INFO (decl));